@@ -58,6 +58,10 @@ impl PyJid {
         }
     }
 
+    fn __bool__(&self) -> bool {
+        self.jid.is_some()
+    }
+
     /*
     // TODO: implement or remove from the API
     fn unescape() {
@@ -205,8 +209,8 @@ impl PyJid {
     /// Use the full JID as the representation.
     fn __repr__(&self) -> String {
         match &self.jid {
-            None => String::new(),
-            Some(jid) => jid.to_string(),
+            None => String::from("JID()"),
+            Some(jid) => format!("JID('{jid}')"),
         }
     }
 
@@ -222,16 +226,20 @@ impl PyJid {
                 Err(_) => return Ok(false),
             }
         };
-        match (&self.jid, &other.jid) {
-            (None, None) => Ok(true),
-            (Some(jid), Some(other)) => match op {
-                pyo3::basic::CompareOp::Eq => Ok(jid == other),
-                pyo3::basic::CompareOp::Ne => Ok(jid != other),
-                _ => Err(PyNotImplementedError::new_err(
-                    "Only == and != are implemented",
-                )),
-            },
-            _ => Ok(false),
+        match op {
+            pyo3::basic::CompareOp::Eq => Ok(match (&self.jid, &other.jid) {
+                (None, None) => true,
+                (Some(jid), Some(other)) => jid == other,
+                _ => false,
+            }),
+            pyo3::basic::CompareOp::Ne => Ok(match (&self.jid, &other.jid) {
+                (None, None) => false,
+                (Some(jid), Some(other)) => jid != other,
+                _ => true,
+            }),
+            _ => Err(PyNotImplementedError::new_err(
+                "Only == and != are implemented",
+            )),
         }
     }
 
@@ -245,14 +253,14 @@ impl PyJid {
         }
     }
 
-    fn __getstate__(&self, py: Python) -> PyResult<Option<PyObject>> {
+    fn __getstate__(&self, py: Python) -> PyResult<Option<Py<PyAny>>> {
         match &self.jid {
             Some(jid) => Ok(Some(PyString::new(py, jid.as_str()).into())),
             None => Ok(None)
         }
     }
 
-    fn __setstate__(&mut self, py: Python, state: PyObject) -> PyResult<()> {
+    fn __setstate__(&mut self, py: Python, state: Py<PyAny>) -> PyResult<()> {
         if state.is_none(py) {
             self.jid = None;
         } else {
