@@ -120,11 +120,14 @@ PLUGINS = [
     'xep_0444',  # Message Reactions
     'xep_0446',  # File metadata element
     'xep_0447',  # Stateless file sharing
+    'xep_0455',  # Service Outage Status
     'xep_0461',  # Message Replies
     'xep_0469',  # Bookmarks Pinning
     'xep_0482',  # Call Invites
     'xep_0490',  # Message Displayed Synchronization
     'xep_0492',  # Chat Notification Settings
+    'xep_0494',  # Client Access Management
+    'xep_0502',  # MUC Activity Indicator
     # Meant to be imported by plugins
 ]
 
