@@ -24,6 +24,6 @@ class XEP_0421(BasePlugin):
         register_stanza_plugin(Message, OccupantId)
         register_stanza_plugin(Presence, OccupantId)
 
-    async def has_feature(self, jid: JID) -> bool:
-        info = await self.xmpp['xep_0030'].get_info(jid)
+    async def has_feature(self, jid: JID, **discokwargs) -> bool:
+        info = await self.xmpp['xep_0030'].get_info(jid, **discokwargs)
         return self.namespace in info.get_features()
