@@ -1,13 +1,19 @@
-"""
-    Slixmpp: The Slick XMPP Library
-    Copyright (C) 2010  Nathanael C. Fritz
-    This file is part of Slixmpp.
 
-    See the file LICENSE for copying permission.
-"""
+# Slixmpp: The Slick XMPP Library
+# Copyright (C) 2010  Nathanael C. Fritz
+# This file is part of Slixmpp.
+# See the file LICENSE for copying permission.
+
+from slixmpp.stanza import Presence
+from slixmpp.types import RosterState, ResourceDict, RosterDBProtocol, JidStr
+
+from typing import TYPE_CHECKING, Optional, Dict, Any, List, Union
+
+if TYPE_CHECKING:
+    from slixmpp import BaseXMPP
 
 
-class RosterItem(object):
+class RosterItem:
 
     """
     A RosterItem is a single entry in a roster node, and tracks
@@ -104,19 +110,27 @@ class RosterItem(object):
         handle_probe        -- Handle a presence probe query.
     """
 
-    def __init__(self, xmpp, jid, owner=None,
-                 state=None, db=None, roster=None):
+    xmpp: 'BaseXMPP'
+    jid: JidStr
+    owner: JidStr
+    db: Optional[RosterDBProtocol]
+    _state: RosterState
+    resources: Dict[str, ResourceDict]
+    _db_state: Dict[str, Any]
+    last_status: Optional[Presence]
+
+    def __init__(self, xmpp: 'BaseXMPP', jid: JidStr, owner: Optional[JidStr] = None,
+                 state: Optional[RosterState] = None, db: Optional[RosterDBProtocol] = None, roster=None):
         """
         Create a new roster item.
 
-        Arguments:
-            xmpp   -- The main Slixmpp instance.
-            jid    -- The item's JID.
-            owner  -- The roster owner's JID. Defaults
+        :param xmpp: The main Slixmpp instance.
+        :param jid: The item's JID.
+        :param owner: The roster owner's JID. Defaults
                       so self.xmpp.boundjid.bare.
-            state  -- A dictionary of initial state values.
-            db     -- An optional interface to an external datastore.
-            roster -- The roster object containing this entry.
+        :param state: A dictionary of initial state values.
+        :param db: An optional interface to an external datastore.
+        :param roster: The roster object containing this entry.
         """
         self.xmpp = xmpp
         self.jid = jid
@@ -125,7 +139,10 @@ class RosterItem(object):
         self.resources = {}
         self.roster = roster
         self.db = db
-        self._state = state or {
+        if state:
+            self._state = state
+        else:
+            self._state = RosterState({
                 'from': False,
                 'to': False,
                 'pending_in': False,
@@ -133,26 +150,27 @@ class RosterItem(object):
                 'whitelisted': False,
                 'subscription': 'none',
                 'name': '',
-                'groups': []}
+                'groups': [],
+                'removed': False,
+            })
 
         self._db_state = {}
         self.load()
 
-    def set_backend(self, db=None, save=True):
+    def set_backend(self, db: Optional[RosterDBProtocol]=None, save: bool = True) -> None:
         """
         Set the datastore interface object for the roster item.
 
-        Arguments:
-            db   -- The new datastore interface.
-            save -- If True, save the existing state to the new
-                    backend datastore. Defaults to True.
+        :param db: The new datastore interface.
+        :param save: If True, save the existing state to the new
+                     backend datastore. Defaults to True.
         """
         self.db = db
         if save:
             self.save()
         self.load()
 
-    def load(self):
+    def load(self) -> Optional[RosterState]:
         """
         Load the item's state information from an external datastore,
         if one has been provided.
@@ -172,13 +190,12 @@ class RosterItem(object):
             return self._state
         return None
 
-    def save(self, remove=False):
+    def save(self, remove: bool = False) -> None:
         """
         Save the item's state information to an external datastore,
         if one has been provided.
 
-        Arguments:
-            remove -- If True, expunge the item from the datastore.
+        :param remove: If True, expunge the item from the datastore.
         """
         self['subscription'] = self._subscription()
         if remove:
@@ -200,7 +217,7 @@ class RosterItem(object):
         else:
             raise KeyError
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Union[str, List[str], bool]) -> None:
         """
         Set the value of a state field.
 
@@ -213,10 +230,10 @@ class RosterItem(object):
         """
         if key in self._state:
             if key in ['name', 'subscription', 'groups']:
-                self._state[key] = value
+                self._state[key] = value  # type: ignore
             else:
                 value = str(value).lower()
-                self._state[key] = value in ('true', '1', 'on', 'yes')
+                self._state[key] = value in ('true', '1', 'on', 'yes')  # type: ignore
         else:
             raise KeyError
 
@@ -231,7 +248,7 @@ class RosterItem(object):
         else:
             return 'none'
 
-    def remove(self):
+    def remove(self) -> None:
         """
         Remove a JID's whitelisted status and unsubscribe if a
         subscription exists.
@@ -247,7 +264,7 @@ class RosterItem(object):
         self['whitelisted'] = False
         self.save()
 
-    def subscribe(self):
+    def subscribe(self) -> None:
         """Send a subscription request to the JID."""
         p = self.xmpp.Presence()
         p['to'] = self.jid
@@ -258,7 +275,7 @@ class RosterItem(object):
         self.save()
         p.send()
 
-    def authorize(self):
+    def authorize(self) -> None:
         """Authorize a received subscription request from the JID."""
         self['from'] = True
         self['pending_in'] = False
@@ -266,7 +283,7 @@ class RosterItem(object):
         self._subscribed()
         self.send_last_presence()
 
-    def unauthorize(self):
+    def unauthorize(self) -> None:
         """Deny a received subscription request from the JID."""
         self['from'] = False
         self['pending_in'] = False
@@ -279,7 +296,7 @@ class RosterItem(object):
             p['from'] = self.owner
         p.send()
 
-    def _subscribed(self):
+    def _subscribed(self) -> None:
         """Handle acknowledging a subscription."""
         p = self.xmpp.Presence()
         p['to'] = self.jid
@@ -288,7 +305,7 @@ class RosterItem(object):
             p['from'] = self.owner
         p.send()
 
-    def unsubscribe(self):
+    def unsubscribe(self) -> None:
         """Unsubscribe from the JID."""
         p = self.xmpp.Presence()
         p['to'] = self.jid
@@ -298,7 +315,7 @@ class RosterItem(object):
         self.save()
         p.send()
 
-    def _unsubscribed(self):
+    def _unsubscribed(self) -> None:
         """Handle acknowledging an unsubscribe request."""
         p = self.xmpp.Presence()
         p['to'] = self.jid
@@ -315,15 +332,14 @@ class RosterItem(object):
         Otherwise, forward the send request to the recipient's roster
         entry for processing.
 
-        Arguments:
-            pshow     -- The presence's show value.
-            pstatus   -- The presence's status message.
-            ppriority -- This connections' priority.
-            pto       -- The recipient of a directed presence.
-            pfrom     -- The sender of a directed presence, which should
-                         be the owner JID plus resource.
-            ptype     -- The type of presence, such as 'subscribe'.
-            pnick     -- Optional nickname of the presence's sender.
+        :param pshow: The presence's show value.
+        :param pstatus: The presence's status message.
+        :param ppriority: This connections' priority.
+        :param pto: The recipient of a directed presence.
+        :param pfrom: The sender of a directed presence, which should
+                      be the owner JID plus resource.
+        :param ptype: The type of presence, such as 'subscribe'.
+        :param pnick: Optional nickname of the presence's sender.
         """
         if self.xmpp.is_component and not kwargs.get('pfrom', ''):
             kwargs['pfrom'] = self.owner
@@ -331,7 +347,7 @@ class RosterItem(object):
             kwargs['pto'] = self.jid
         self.xmpp.send_presence(**kwargs)
 
-    def send_last_presence(self):
+    def send_last_presence(self) -> None:
         if self.last_status is None:
             pres = self.roster.last_status
             if pres is None:
@@ -346,11 +362,13 @@ class RosterItem(object):
         else:
             self.last_status.send()
 
-    def handle_available(self, presence):
+    def handle_available(self, presence: Presence) -> None:
         resource = presence['from'].resource
-        data = {'status': presence['status'],
-                'show': presence['show'],
-                'priority': presence['priority']}
+        data = ResourceDict({
+            'status': presence['status'],
+            'show': presence['show'],
+            'priority': presence['priority'],
+        })
         got_online = not self.resources
         if resource not in self.resources:
             self.resources[resource] = {}
@@ -362,7 +380,7 @@ class RosterItem(object):
         if old_show != presence['show'] or old_status != presence['status']:
             self.xmpp.event('changed_status', presence)
 
-    def handle_unavailable(self, presence):
+    def handle_unavailable(self, presence: Presence) -> None:
         resource = presence['from'].resource
         if not self.resources:
             return
@@ -372,7 +390,7 @@ class RosterItem(object):
         if not self.resources:
             self.xmpp.event('got_offline', presence)
 
-    def handle_subscribe(self, presence):
+    def handle_subscribe(self, presence: Presence) -> None:
         """
         +------------------------------------------------------------------+
         |  EXISTING STATE          |  DELIVER?  |  NEW STATE               |
@@ -397,9 +415,11 @@ class RosterItem(object):
             self.save()
         else:
             #server shouldn't send an invalid subscription request
+            if not self['from'] and not self['pending_in']:
+                self['pending_in'] = True
             self.xmpp.event('roster_subscription_request', presence)
 
-    def handle_subscribed(self, presence):
+    def handle_subscribed(self, presence: Presence) -> None:
         """
         +------------------------------------------------------------------+
         |  EXISTING STATE          |  DELIVER?  |  NEW STATE               |
@@ -424,7 +444,7 @@ class RosterItem(object):
         else:
             self.xmpp.event('roster_subscription_authorized', presence)
 
-    def handle_unsubscribe(self, presence):
+    def handle_unsubscribe(self, presence: Presence) -> None:
         """
         +------------------------------------------------------------------+
         |  EXISTING STATE          |  DELIVER?  |  NEW STATE               |
@@ -441,7 +461,7 @@ class RosterItem(object):
         +------------------------------------------------------------------+
         """
         if self.xmpp.is_component:
-            if not self['from']  and self['pending_in']:
+            if not self['from'] and self['pending_in']:
                 self['pending_in'] = False
                 self._unsubscribed()
             elif self['from']:
@@ -450,9 +470,13 @@ class RosterItem(object):
                 self.xmpp.event('roster_subscription_remove', presence)
             self.save()
         else:
+            if not self['from'] and self['pending_in']:
+                self['pending_in'] = False
+            elif self['from']:
+                self['from'] = False
             self.xmpp.event('roster_subscription_remove', presence)
 
-    def handle_unsubscribed(self, presence):
+    def handle_unsubscribed(self, presence: Presence) -> None:
         """
         +------------------------------------------------------------------+
         |  EXISTING STATE          |  DELIVER?  |  NEW STATE               |
@@ -478,7 +502,7 @@ class RosterItem(object):
         else:
             self.xmpp.event('roster_subscription_removed', presence)
 
-    def handle_probe(self, presence):
+    def handle_probe(self, presence: Presence) -> None:
         if self['from']:
             self.send_last_presence()
         if self['pending_out']:
@@ -486,12 +510,12 @@ class RosterItem(object):
         if not self['from']:
             self._unsubscribed()
 
-    def reset(self):
+    def reset(self) -> None:
         """
         Forgot current resource presence information as part of
         a roster reset request.
         """
         self.resources = {}
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return repr(self._state)
